class LocalRatingsPage
{

    constructor()
    {
        // GUI objects
        this.searchPlayerInput = Engine.GetGUIObjectByName("searchPlayerInput");
        this.infoButton = Engine.GetGUIObjectByName("infoButton");
        this.tabEvolutionChart = Engine.GetGUIObjectByName("tabEvolutionChart");
        this.tabEvolutionChartButton = Engine.GetGUIObjectByName("tabEvolutionChartButton");
        this.tabCivilizationChart = Engine.GetGUIObjectByName("tabCivilizationChart");
        this.tabCivilizationChartButton = Engine.GetGUIObjectByName("tabCivilizationChartButton");
        this.tabDividerLeft = Engine.GetGUIObjectByName("tabDividerLeft");
        this.tabDividerRight = Engine.GetGUIObjectByName("tabDividerRight");
        this.playerList = Engine.GetGUIObjectByName("playerList");
        this.mainMenuButton = Engine.GetGUIObjectByName("mainMenuButton");
        this.optionsButton = Engine.GetGUIObjectByName("optionsButton");
        this.rebuildButton = Engine.GetGUIObjectByName("rebuildButton");

        // Events
        this.searchPlayerInput.onTextEdit = this.onSearchPlayerInputTextEdit.bind(this);
        this.infoButton.onPress = this.onInfoButtonPress.bind(this);
        this.tabEvolutionChartButton.onPress = this.onTabEvolutionChartButtonPress.bind(this);
        this.tabCivilizationChartButton.onPress = this.onTabCivilizationChartButtonPress.bind(this);
        this.playerList.onSelectionColumnChange = this.onSelectionColumnChange.bind(this);
        this.playerList.onSelectionChange = this.onSelectionChange.bind(this);
        this.mainMenuButton.onPress = this.onMainMenuButtonPress.bind(this);
        this.optionsButton.onPress = this.onOptionsButtonPress.bind(this);
        this.rebuildButton.onPress = this.onRebuildButtonPress.bind(this);

        // Initialize databases
        this.ratingsDatabase = {};
        this.historyDatabase = {};
        this.initializeDatabases();

        this.columns = {};
        this.currentChart;
        this.currentPlayerProfile;
        this.tabs = [this.tabEvolutionChart, this.tabCivilizationChart];
        this.selectedTab = this.tabs[0];
        this.updateTabs();
    }

    // searchPlayerInput events

    onSearchPlayerInputTextEdit()
    {
        this.updateList();
    }

    // infoButton events

    onInfoButtonPress()
    {
        Engine.PushGuiPage("page_localratings_about.xml");
    }

    // tab buttons event

    onTabEvolutionChartButtonPress()
    {
        this.selectedTab = this.tabEvolutionChart;
        this.updateTabs();
        if (this.currentChart)
            this.currentChart.clear();
        this.showChart();
    }

    onTabCivilizationChartButtonPress()
    {
        this.selectedTab = this.tabCivilizationChart;
        this.updateTabs();
        if (this.currentChart)
            this.currentChart.clear();
        this.showChart();
    }

    // playerList events

    onSelectionColumnChange()
    {
        this.updateList();
    }

    onSelectionChange()
    {
        if (this.currentChart)
            this.currentChart.clear();

        if (this.currentPlayerProfile)
            this.currentPlayerProfile.clear();

        this.showChart();
        this.showPlayerProfile();
    }

    // Button events

    onMainMenuButtonPress()
    {
        Engine.SwitchGuiPage("page_pregame.xml");
    }

    onOptionsButtonPress()
    {
        Engine.PushGuiPage("page_localratings_options.xml", {}, (data) => {
            // If the two databases are defined, then options have been already saved
            if (data.ratingsDatabase && data.historyDatabase)
            {
                // Save globals
                this.ratingsDatabase = data.ratingsDatabase;
                this.historyDatabase = data.historyDatabase;

                // Save ratings and history databases
                let ratingsDB = new LocalRatingsRatingsDB();
                ratingsDB.ratingsDatabase = this.ratingsDatabase;
                ratingsDB.historyDatabase = this.historyDatabase;
                ratingsDB.save();
            }

            // Update list if needed
            if (Object.keys(data.changes).some(x => x.startsWith("localratings.filter.") || x.startsWith("localratings.weight.") || x.startsWith("localratings.playerfilter.")))
                this.updateList();

            // If charts options have changed, update charts
            if (Object.keys(data.changes).some(x => x.startsWith("localratings.charts.")) && this.selectedTab.name == "tabEvolutionChart")
                this.showChart();
        });
    }

    onRebuildButtonPress()
    {
        // Update the replay database with new replays
        let replayDB = new LocalRatingsReplayDB();
        replayDB.rebuild();

        // Update ratings and history databases
        let ratingsDB = new LocalRatingsRatingsDB();
        ratingsDB.rebuild();
        ratingsDB.save();

        // Update globals
        this.ratingsDatabase = ratingsDB.ratingsDatabase;
        this.historyDatabase = ratingsDB.historyDatabase;

        // Push to GUI
        this.updateList();
    }

    // Non-event functions

    initializeDatabases()
    {
        let ratingsDB = new LocalRatingsRatingsDB();
        ratingsDB.load();
        this.ratingsDatabase = ratingsDB.ratingsDatabase;
        this.historyDatabase = ratingsDB.historyDatabase;
    }

    updateTabs()
    {
        let tab = this.selectedTab;
        this.tabs.forEach(x => x.sprite = "ModernTabHorizontalBackground");
        tab.sprite = "ModernTabHorizontalForeground";

        let tabDividerLeftSize = this.tabDividerLeft.size;
        tabDividerLeftSize.rright = tab.size.rleft;
        this.tabDividerLeft.size = tabDividerLeftSize;

        let tabDividerRightSize = this.tabDividerRight.size;
        tabDividerRightSize.rleft = tab.size.rright;
        this.tabDividerRight.size = tabDividerRightSize;
    }

    showChart()
    {
        if (this.playerList.selected == -1)
            return;

        const player = this.playerList.list_playername[this.playerList.selected];
        const playerData = this.historyDatabase[player];
        if (this.selectedTab.name == "tabEvolutionChart")
            this.currentChart = new LocalRatingsEvolutionChart(player, playerData);
        else if (this.selectedTab.name == "tabCivilizationChart")
            this.currentChart = new LocalRatingsCivilizationChart(player, playerData);
        this.currentChart.draw();
    }

    showPlayerProfile()
    {
        if (this.playerList.selected == -1)
            return;

        const player = this.playerList.list_playername[this.playerList.selected];
        const rank = this.playerList.list_rank[this.playerList.selected];
        const playerData = this.historyDatabase[player];
        this.currentPlayerProfile = new LocalRatingsPlayerProfile(player, playerData, rank);
        this.currentPlayerProfile.fill();
    }

    getColumns()
    {
        // Filter players
        const filterObj = new LocalRatingsPlayerFilter(this.ratingsDatabase);
        const filteredRatings = Object.entries(this.ratingsDatabase).filter(([key, value]) => !filterObj.applies(key));
        const filteredRatingsDatabase = Object.fromEntries(filteredRatings);

        // Create an array of players sorted by rank
        let sortedByRank = Object.keys(filteredRatingsDatabase).map(x => [
            x,
            filteredRatingsDatabase[x].rating,
            filteredRatingsDatabase[x].matches
        ]);
        sortedByRank.sort((a, b) => b[1] - a[1]);

        // Return JSON rating database into column format
        return {
            "rank": sortedByRank.map((x, i) => i+1),
            "player": sortedByRank.map(x => PlayerColor.ColorPlayerName(x[0], "", "")),
            "rating": sortedByRank.map(x => formatRating_LocalRatings(x[1])),
            "matches": sortedByRank.map(x => x[2]),
            "playername": sortedByRank.map(x => x[0])
        };
    }

    sortBySelectedColumn()
    {
        let rows = this.columns.rank.map((x, i) => [
            x,
            this.columns.player[i],
            this.columns.rating[i],
            this.columns.matches[i],
            this.columns.playername[i]
        ]);
        const column = this.playerList.selected_column;
        if (column == "rank" || column == "rating")
            rows.sort((a, b) => b[0]-a[0]);
        else if (column == "matches")
            rows.sort((a, b) => (b[3] == a[3]) ? b[0]-a[0] : b[3]-a[3]);
        else
            rows.sort((a, b) => b[4].localeCompare(a[4]));
        if (this.playerList.selected_column_order < 0)
            rows.reverse();
        this.columns = {
            "rank": rows.map(x => x[0]),
            "player": rows.map(x => x[1]),
            "rating": rows.map(x => x[2]),
            "matches": rows.map(x => x[3]),
            "playername": rows.map(x => x[4])
        };
    }

    filterBySearchPlayerInput()
    {
        const text = this.searchPlayerInput.caption.toLowerCase();
        const filterPlayer = (pos, text) => this.columns.player[pos].toLowerCase().includes(text);
        return {
            "rank": this.columns.rank.filter((x, i) => filterPlayer(i, text)),
            "player": this.columns.player.filter((x, i) => filterPlayer(i, text)),
            "rating": this.columns.rating.filter((x, i) => filterPlayer(i, text)),
            "matches": this.columns.matches.filter((x, i) => filterPlayer(i, text)),
            "playername": this.columns.playername.filter((x, i) => filterPlayer(i, text))
        };
    }

    populate(columns)
    {
        this.playerList.selected = -1;
        this.playerList.list_rank = columns.rank;
        this.playerList.list_player = columns.player;
        this.playerList.list_rating = columns.rating;
        this.playerList.list_matches = columns.matches;
        this.playerList.list_playername = columns.playername;
        // Change these last, otherwise crash
        this.playerList.list = columns.player;
        this.playerList.list_data = columns.player;
    }

    updateList()
    {
        // If no columns, return
        this.columns = this.getColumns();
        if (Object.keys(this.columns).length == 0)
            return;

        // Populate table with columns
        this.sortBySelectedColumn();
        const columns = this.filterBySearchPlayerInput();
        this.populate(columns);
    }

}
